@@ -20,19 +20,29 @@
 # the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 DATE=`date`
-MAIL_AGENT=/bin/mail
-GNATS_ROOT=@GNATS_ROOT@
-LIBDIR=@LIBDIR@
+GNATS_ROOT=xGNATS_ROOTx
+LIBDIR=xLIBDIRx
 PATH=$PATH:$GNATS_ROOT/gnats-bin
 PR_EDIT=$LIBDIR/gnats/pr-edit
-GNATS_ADDR=$LIBDIR/gnats/pr-addr
+PR_ADDR=$LIBDIR/gnats/pr-addr
 locked=
+version=xVERSIONx
 
 usage="Usage: $0 [-hV] [--version] [--help] PR"
-version=@VERSION@
+
+# Newer config information?
+[ -f ${GNATS_ROOT}/gnats-adm/config ] && . ${GNATS_ROOT}/gnats-adm/config
+
+# Host-specific; must come after config file.
+MAIL_AGENT="xMAIL_AGENTx"
 
 # check to see if there is a $EDITOR; if not, use vi
-[ -z "$EDITOR" ] && EDITOR=vi
+[ -z "$VISUAL" ] &&
+  if [ -z "$EDITOR" ]; then
+    VISUAL=vi
+  else
+    VISUAL="$EDITOR"
+  fi
 
 # parse command line.
 # for the non-flag argument, assume it's correct.  if it's a full
@@ -102,7 +112,9 @@ if [ -z "$HOSTNAME" ]; then
 fi
 
 if [ -n "$HOSTNAME" ]; then
-  me="$me@$HOSTNAME"
+  full_me="$me@$HOSTNAME"
+else
+  full_me="$me"
 fi
 
 # now we have a valid $full_id.. use its full path
@@ -114,7 +126,7 @@ new="/tmp/ep$$"
 cp $pr $new
 
 # lock the pr
-$PR_EDIT --lock $me $full_id 2> /tmp/u$$
+$PR_EDIT --lock $full_me $full_id 2> /tmp/u$$
 locked=t
 
 if [ -s /tmp/u$$ ]; then
@@ -129,7 +141,7 @@ fi
 
 # here's where we actually call the editor.
 cp $new $new.old
-$EDITOR $new
+$VISUAL $new
 if cmp -s $new.old $new ; then
   echo "edit-pr: PR not changed"
   $PR_EDIT --unlock $full_id
@@ -138,7 +150,7 @@ fi
 rm -f $new.old
 
 # error-check output by calling pr-edit --check; if mistakes exist,
-# call $EDITOR or exit
+# call $VISUAL or exit
 checking=t
 while [ "$checking" != "" ]; do
   errors="`$PR_EDIT --check < $new`"
@@ -153,7 +165,7 @@ while [ "$checking" != "" ]; do
         exit 0
         ;;
     esac
-    $EDITOR $new
+    $VISUAL $new
   else
     checking=
   fi
@@ -164,12 +176,22 @@ done
 # add audit trail
 # mail changes to relevant parties
 
-old_state=`grep '^>State:' $pr | sed -e 's,^>State: ,,g' -e 's, ,,g'`
-new_state=`grep '^>State:' $new | sed -e 's,^>State: ,,g' -e 's, ,,g'`
-old_resp=`grep '^>Responsible:' $pr | sed -e 's,^>Responsible: ,,g' -e 's, ,,g'`
-new_resp=`grep '^>Responsible:' $new | sed -e 's,^>Responsible: ,,g' -e 's, ,,g'`
-old_synopsis=`grep '^>Synopsis:' $pr | sed -e 's,^>Synopsis: ,,g' -e 's,^ *,,'`
-new_synopsis=`grep '^>Synopsis:' $new | sed -e 's,^>Synopsis: ,,g' -e 's,^ *,,'`
+old_state="`sed -n '/^>State:/{s,^>[-a-zA-Z]*: *,,;p;q;}' $pr`"
+new_state="`sed -n '/^>State:/{s,^>[-a-zA-Z]*: *,,;p;q;}' $new`"
+old_resp="`sed -n '/^>Responsible:/{s,^>[-a-zA-Z]*: *,,;s, *(.*,,g;p;q;}' $pr`"
+new_resp="`sed -n '/^>Responsible:/{s,^>[-a-zA-Z]*: *,,;s, *(.*,,g;p;q;}' $new`"
+old_synopsis="`sed -n '/^>Synopsis:/{s,^>[-a-zA-Z]*: *,,;p;q;}' $pr`"
+new_synopsis="`sed -n '/^>Synopsis:/{s,^>[-a-zA-Z]*: *,,;p;q;}' $new`"
+
+# If you can read this, you may have a future in sed(1) programming.
+reply_to="`sed -n \
+-e '/^$/{g;s/	/ /g;s/\n/ /g;s/^.*: *//;s/ *(.*) *//;s/.*<//;s/>.*//;p;q;}' \
+-e '/^Reply-To:/h' \
+-e '/^Reply-To:/,/^[^ 	]/{s/^[^ 	].*//;H;}' \
+-e '/^Reply-To:/,$b' \
+-e '/^From:/h' \
+-e '/^From:/,/^[^ 	]/{s/^[^ 	].*//;H;}' \
+$pr`"
 
 change_msg=/tmp/ed_pr_ch$$
 
@@ -181,38 +203,37 @@ if [ "$old_resp" != "$new_resp" ]; then
   resp_change=yes
 fi
 
-if [ ! -z "$state_change" -o ! -z "$resp_change" ]; then  # we've got a change
+if [ ! -z "$state_change" ] || [ ! -z "$resp_change" ]; then
+  # we've got a change
   mail_to="$me"
   if [ ! -z "$state_change" ]; then
     echo State-Changed-From-To: "$old_state"-"$new_state" >> $change_msg
     echo State-Changed-By: $me >> $change_msg
     echo State-Changed-When: $DATE >> $change_msg
     echo "State-Changed-Why: " >> $change_msg
-    echo "Why did the state change? (Ctrl-D to end)"
+    echo 'Why did the state change? (Ctrl-D to end)'
     cat >> $change_msg
+    to_old=1
+    to_subm=1
   fi
-  if [ ! -z "$state_change" -a ! -z "$resp_change" ]; then
+  if [ ! -z "$state_change" ] && [ ! -z "$resp_change" ]; then
     echo "" >> $change_msg
     echo "" >> $change_msg
   fi
   if [ ! -z "$resp_change" ]; then
-    echo Responsible-Changed-From-To: $old_resp->$new_resp" >> $change_msg
+    echo Responsible-Changed-From-To: "$old_resp"'->'"$new_resp" >> $change_msg
     echo Responsible-Changed-By: $me >> $change_msg
     echo Responsible-Changed-When: $DATE >> $change_msg
     echo "Responsible-Changed-Why: " >> $change_msg
-    echo "Why did the responsible person change? (Ctrl-D to end)"
+    echo 'Why did the responsible person change? (Ctrl-D to end)'
     cat >> $change_msg
-    if [ "$old_resp" != "$me" ]; then
-      mail_to="$mail_to `$GNATS_ADDR $old_resp`"
-    fi
-    if [ "$new_resp" != "$me" ]; then
-      mail_to="$mail_to `$GNATS_ADDR $new_resp`"
-    fi
-  else
-    mail_to="$mail_to `$GNATS_ADDR $old_resp`"
+    to_old=1
+    to_new=1
   fi
 
-  echo edit-pr: filing $full_id back into the database
+  if [ -z "$to_subm" ]; then mail_to="${reply_to}, ${mail_to}" ; fi
+  if [ -z "$to_old" ] ; then mail_to="${mail_to}, `$PR_ADDR ${old_resp}`" ; fi
+  if [ -z "$to_new" ] ; then mail_to="${mail_to}, `$PR_ADDR ${new_resp}`" ; fi
 
   # We have to quote the end of each line in the change_msg, so that sed
   # won't try to use it as a new command.
@@ -221,7 +242,7 @@ if [ ! -z "$state_change" -o ! -z "$resp_change" ]; then  # we've got a change
 " $new > $new.tmp
   mv -f $new.tmp $new
 
-  $MAIL_AGENT $mail_to << __EOF__
+  $MAIL_AGENT << __EOF__
 To: $mail_to
 From: $me
 Subject: Changed information for PR $full_id
@@ -237,6 +258,8 @@ fi`
 __EOF__
 fi
 
+echo "edit-pr: filing $full_id back into the database"
+
 # call PR_EDIT on the new file and clean up
 $PR_EDIT < $new
 $PR_EDIT --unlock $full_id
