@@ -19,6 +19,8 @@ You should have received a copy of the GNU General Public License
 along with GNU GNATS; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <sys/types.h>
 
@@ -30,20 +32,20 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #endif
 #include <sys/stat.h>		/* for stat(2) */
 #include <time.h>
-#include <sys/types.h>
 #include <signal.h>
+#include <ctype.h>
 
-#include "config.h"
 #include "gnats.h"
 #include "globals.h"
 #include "pathmax.h"
 #include "headers.h"
+#include "regex.h"
 
 static int verify_analysis	PARAMS((void));
 static void run_atpr		PARAMS((Submitter *, struct tm *, char *, char *));
 static void free_category	PARAMS((Category *));
 static void free_submitter	PARAMS((Submitter *));
-static void free_responsible		PARAMS((Responsible *));
+static void free_responsible	PARAMS((Responsible *));
 static void reply_to_submitter	PARAMS((char *));
 static void create_report	PARAMS((char *, int));
 static int get_bug_number	PARAMS((void));
@@ -63,8 +65,8 @@ gnats (fp)
   char number[14]; /* No more than 14 digits in a PR #, ha ha.  */
   char bug_name[STR_MAX];
   Responsible *responsible = NULL;
-  char arrival_time[26], response_time[26];
-  struct tm *expired;
+  char arrival_time[26];
+  struct tm *expired = NULL;
 
   char *site, *bug_group;
   char *subject = NULL, *synopsis = NULL;
@@ -93,12 +95,12 @@ gnats (fp)
   if (find_submitter (&submitter, site) == -1)
     {
       log_msg (LOG_INFO, 1, "resetting to default submitter from:", site);
-      site = DEFAULT_SUBMITTER;
-      set_field (SUBMITTER, DEFAULT_SUBMITTER);
+      site = def_subm;
+      set_field (SUBMITTER, def_subm);
 
       /* If it's still an error, punt because this should not happen.  */
-      if (find_submitter (&submitter, DEFAULT_SUBMITTER) == -1)
-        punt (1, "can not find default submitter %s", DEFAULT_SUBMITTER);
+      if (find_submitter (&submitter, def_subm) == -1)
+        punt (1, "can not find default submitter %s", def_subm);
     }
 
   bug_group = field_value (CATEGORY);
@@ -148,15 +150,15 @@ gnats (fp)
 	{
 	  log_msg (LOG_INFO, 1, "responsible entry not found for",
 		   category.person);
-	  responsible = get_responsible_address (GNATS_ADMIN);
+	  responsible = get_responsible_address (gnats_admin);
 	  if (responsible == NULL)
 	    punt (0, "can't find gnats-admin in responsibles file. ");
 	}
  
       if (responsible->fullname)
-	sprintf (path, "%s (%s)", responsible->alias, responsible->fullname);
+	sprintf (path, "%s (%s)", responsible->key, responsible->fullname);
       else
-	sprintf (path, "%s", responsible->alias);
+	sprintf (path, "%s", responsible->key);
 
       set_field (RESPONSIBLE, path);
       log_msg (LOG_INFO, 1, "responsible person is:", path);
@@ -167,8 +169,7 @@ gnats (fp)
      there is nothing there. */
   synopsis = field_value (SYNOPSIS);
   subject = header_value (SUBJECT);
-  if ((subject == NULL || *subject == '\0')
-      && (synopsis != NULL && *synopsis != '\0'))
+  if (*subject == '\0' && *synopsis != '\0')
     {
       /* The `Subject:' line is assumed to have a newline at the end.  */
       int l = strlen (synopsis);
@@ -179,8 +180,7 @@ gnats (fp)
       set_header (SUBJECT, buf);
     }
 
-  if ((synopsis == NULL || *synopsis == '\0')
-      && (subject != NULL && *subject != '\0'))
+  if (*synopsis == '\0' && *subject != '\0')
     {
       /* The synopsis shouldn't have a newline at the end.  */
       char *l;
@@ -201,10 +201,8 @@ gnats (fp)
 
   /* If the submitter isn't supposed to have a response time, don't bother
      trying to get it.  */
-  if (submitter.rtime == -1)
-    response_time[0] = '\0';
-  else
-    expired = get_response_time (submitter.rtime, &response_time[0]);
+  if (submitter.rtime >= 0)
+    expired = get_response_time (submitter.rtime);
 
   /* Put together the path to where the bug will be stored.  If the dir
      is not there, and the category is PENDING, auto-create that one,
@@ -289,7 +287,7 @@ gnats (fp)
 	run_atpr (&submitter, expired, bug_name, path);
     }
   else
-    notify_responsible (GNATS_ADMIN, NULL, NULL, NULL, NULL, 0, (struct tm *)0);
+    notify_responsible (gnats_admin, NULL, NULL, NULL, NULL, 0, (struct tm *)0);
 
   free_category (&category);
   free_submitter (&submitter);
@@ -305,6 +303,32 @@ verify_analysis ()
 	&& strcmp (severity, "serious") == 0);
 }
 
+static char *
+arg_quote (s)
+     char *s;
+{
+  /* 2x seems a reasonable amount, though 4x is needed for complete safety.  */
+  char *buf = xmalloc (2 * strlen (s));
+  char *p;
+
+  for (p = buf; *s; s++, p++)
+    {
+      if (*s == '\'')
+	{
+	  *p++ = '\'';
+	  *p++ = '\\';
+	  *p++ = '\'';
+	}
+      *p = *s;
+    }
+  if (p == buf)
+    *p++ = ' ';
+  
+  *p = '\0';
+
+  return buf;
+}
+
 static void
 run_atpr (submitter, expired, bug_name, path)
      Submitter *submitter;
@@ -317,9 +341,8 @@ run_atpr (submitter, expired, bug_name, path)
   int len, i;
   static char *ats[] = { "/usr/bin/at", "/bin/at", NULL };
 
-  at_pr = (char *) xmalloc (strlen (gnats_root) + strlen (BINDIR) + 7);
-  strcpy (at_pr, gnats_root);
-  strcat (at_pr, BINDIR);
+  at_pr = (char *) xmalloc (strlen (bindir) + 7);
+  strcpy (at_pr, bindir);
   strcat (at_pr, "/at-pr");
 
   len = strftime (buf, STR_MAX, "%H:%M %b %d", expired);
@@ -348,13 +371,23 @@ run_atpr (submitter, expired, bug_name, path)
     }
   else
     {
+      char *orig = arg_quote (field_value (ORIGINATOR));
+      char *cont = arg_quote (submitter->contact);
+      char *pa = arg_quote (gnats_admin);
+      
       fprintf (pipe, "%s %d %s %s %s '%s' '%s' '%s'\n", at_pr,
 	       submitter->rtime, bug_name, path, submitter->key,
-	       field_value (ORIGINATOR), submitter->contact, GNATS_ADMIN);
+	       orig, cont, pa);
+
       pclose (pipe);
+      free (orig);
+      free (cont);
+      free (pa);
     }
 }
 
+static char case_fold[256];
+
 /* Check to see if this message is a reply to an existing PR.  If so,
    return the path to that PR.  */
 char *
@@ -365,38 +398,48 @@ check_if_reply (fp)
   char *path = (char *) xmalloc (PATH_MAX);
   struct stat buf;
   char *s;
-  bool match = FALSE;
+  struct re_pattern_buffer regex;
+  struct re_registers regs;
+  int i, start, end;
 
   s = header_value (SUBJECT);
 
-  if (s == NULL)
+  if (*s == '\0')
     return NULL;
 
-  /* Look for the string.  We assume it's either `category/number: foo',
-     or `Re: category/number: foo'.  Some mailers don't add `Re:' to the
-     beginning of a reply.  */
-  s = get_token (s, token);
+  strcpy (token, s);
+  s = token;
 
-  /* If it has some form of `re:' at the beginning, then get the next
-     token -- it could be the Subject: line that a notification included.  */
-  if (strncasecmp (token, "re:", 3) == 0)
-    s = get_token (s, token);
+  re_set_syntax (RE_NO_BK_PARENS);
+  memset ((void *) &regex, 0, sizeof (regex));
 
-  /* Only check for the PR in the subject if it's got CATEGORY/NUMBER.  */
-  s = (char *) strchr (token, '/');
-  if (s != NULL)
+  if (case_fold[1] == 0)
     {
-      /* Strip off the colon, leaving the category and the PR number.  */
-      s = (char *) strrchr (token, ':');
-      if (s != NULL) 
-        *s = '\0';
-
-      sprintf (path, "%s/%s", gnats_root, token);
-      if (stat (path, &buf) != -1)
-        match = TRUE;
+      int i;
+      for (i=0; i<256; i++)
+	case_fold[i] = tolower(i);
     }
+  regex.translate = case_fold;
 
-  if (! match)
+  /* too restrictive? */
+#define PAT "(.*re[ \t]*:)?[ \t]*([-a-zA-Z0-9_+]*/([0-9]+))"
+  re_compile_pattern (PAT, strlen (PAT), &regex);
+  i = re_match (&regex, s, strlen (s), 0, &regs);
+
+  regfree (&regex);
+
+  if (i <= 0)
+    return NULL;
+
+  start = regs.start[2];
+  end = regs.end[2];
+
+  free (regs.start);
+  free (regs.end);
+  
+  s[end] = '\0';
+  sprintf (path, "%s/%s", gnats_root, s + start);
+  if (stat (path, &buf) == -1)
     return NULL;
 
   return path;
@@ -418,16 +461,16 @@ reply_to_submitter (responsible)
     punt (1, "%s: cannot open mail agent.", program_name);
 
   reply_to = header_value (REPLY_TO);
-  if (reply_to == NULL)
+  if (*reply_to == '\0')
     reply_to = header_value (FROM);
   fprintf (msg, "To: %s", reply_to);
-  fprintf (msg, "From: %s\n", GNATS_ADDR);
+  fprintf (msg, "From: %s\n", gnats_addr);
   subject = header_value (SUBJECT);
   fprintf (msg, "Subject: Re: %s/%s: %s",
 	   field_value (CATEGORY), field_value (NUMBER), subject);
   if (*subject == '\0')
     fprintf (msg, "\n");
-  fprintf (msg, "Reply-To: %s, %s\n", GNATS_ADDR, responsible);
+  fprintf (msg, "Reply-To: %s, %s\n", gnats_addr, responsible);
   fprintf (msg, "In-Reply-To: Your message of %s\t%s\n",
 	   header_value (DATE), header_value (MSG_ID));
 
@@ -564,7 +607,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
 
   {
     char *f = header_value (FROM);
-    if (f)
+    if (*f)
       {
 	fprintf (msg, "From: %s", f);
 	fprintf (msg, "Reply-To: %s", f);
@@ -582,11 +625,9 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
   subject = header_value (SUBJECT);
   fprintf (msg, "Subject: %s/%s: %s",
 	   field_value (CATEGORY), field_value (NUMBER),
-	   subject ? subject : "\n");
-  if (subject && *subject == '\0')
-    fprintf (msg, "\n");
+	   *subject ? subject : "\n");
 
-  if (header_value (DATE))
+  if (*header_value (DATE))
     fprintf (msg, "In-Reply-To: Your message of %s\t%s",
 	     header_value (DATE), header_value (MSG_ID));
 
@@ -666,19 +707,23 @@ append_report (infile, filename)
   int buf_len, buf_max, l;
   Responsible *responsible;
   Submitter submitter;
-  char *addr;
-  char *cc_buf, *cc;
   char *p, *q;
-  int cc_len, cc_max;
   char *from, *to, *subject, *date;
 
   /* Save the values we read while in main().  */
   from = (char *) strdup (header_value (FROM));
   to = (char *) strdup (header_value (TO));
   /* A `Subject:' header is optional.  */
-  if (header_value (SUBJECT))
+  if (*header_value (SUBJECT))
     subject = (char *) strdup (header_value (SUBJECT));
-  date = (char *) strdup (header_value (DATE));
+  if (*header_value (DATE))
+    date = (char *) strdup (header_value (DATE));
+  else
+    {
+      time_t t = time(0);
+      date = (char *) xmalloc (30);
+      strftime (date, 30, "%a, %d %b %y %T %Z\n", localtime(&t));
+    }
 
   /* Now read in the PR itself.  */
   pr_file = fopen (filename, "r+");
@@ -708,7 +753,11 @@ append_report (infile, filename)
   APPEND_STRING ("\n", buf, b, buf_len, buf_max, l, 0);
 
   while ((fgets (line, STR_MAX, infile)) != NULL)
-    APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
+    {
+      /* Quote all correspondence so PR doesn't get messed up */
+      APPEND_STRING (" ", buf, b, buf_len, buf_max, l, 0);
+      APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
+    }
   buf[buf_len] = '\0';
   fclose (infile);
 
@@ -731,24 +780,34 @@ append_report (infile, filename)
     *q = '\0';
 
   responsible = get_responsible_address (p);
-  find_submitter (&submitter, field_value (SUBMITTER));
 
   msg = open_mail_file (responsible->alias);
   if (msg == (FILE *) NULL)
     punt (1, "%s: cannot open mail agent.", program_name);
   fprintf (msg, "To: %s\n", responsible->alias);
 
-  cc = cc_buf = (char *) xmalloc (BUFSIZ);
-  cc_len = 0;
-  cc_max = BUFSIZ;
+  if (find_submitter (&submitter, field_value (SUBMITTER)) == -1)
+    punt (0, "%s: PR %s has a Submitter field of `%s', which is invalid.",
+	  program_name, field_value (NUMBER), field_value (SUBMITTER));
+  else
+    {
+      char *addr;
+      char *cc_buf, *cc;
+      int cc_len, cc_max;
+      
+      cc = cc_buf = (char *) xmalloc (BUFSIZ);
+      cc_len = 0;
+      cc_max = BUFSIZ;
+      
+      *cc = '\0';
+      addr = append_notify (submitter.contact);
+      APPEND_STRING (addr, cc_buf, cc, cc_len, cc_max, l, 1);
+      cc_buf[cc_len] = '\0';
 
-  *cc = '\0';
-  addr = append_notify (submitter.contact);
-  APPEND_STRING (addr, cc_buf, cc, cc_len, cc_max, l, 1);
-  cc_buf[cc_len] = '\0';
-
-  fprintf (msg, "Cc: %s\n", cc_buf);
-  free (cc_buf);
+      fprintf (msg, "Cc: %s\n", cc_buf);
+      free (cc_buf);
+    }
+  
   fprintf (msg, "From: %s", from);
 /*   fprintf (msg, "Resent-From: gnats\n"); */
   fprintf (msg, "Subject: %s", subject);
@@ -827,9 +886,9 @@ free_submitter (submitter)
     free (submitter->fullname);
   if (submitter->type)
     free (submitter->type);
-  if (submitter->contact && submitter->contact != GNATS_ADMIN)
+  if (submitter->contact && submitter->contact != gnats_admin)
     free (submitter->contact);
-  if (submitter->notify && submitter->notify != GNATS_ADMIN)
+  if (submitter->notify && submitter->notify != gnats_admin)
     free (submitter->notify);
   return;
 }
@@ -844,7 +903,7 @@ free_category (category)
     free (category->fullname);
   if (category->person)
     free (category->person);
-  if (category->notify && category->notify != GNATS_ADMIN)
+  if (category->notify && category->notify != gnats_admin)
     free (category->notify);
   return;
 }
@@ -853,11 +912,11 @@ static void
 free_responsible (responsible)
      Responsible *responsible;
 {
-  if (responsible->key && responsible->key != GNATS_ADMIN)
+  if (responsible->key && responsible->key != gnats_admin)
     free (responsible->key);
   if (responsible->fullname)
     free (responsible->fullname);
-  if (responsible->alias && responsible->key != GNATS_ADMIN)
+  if (responsible->alias && responsible->key != gnats_admin)
     free (responsible->alias);
   return;
 }
