@@ -18,6 +18,8 @@ You should have received a copy of the GNU General Public License
 along with GNU GNATS; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <getopt.h>
@@ -99,7 +101,7 @@ do_category (c)
     {
       fprintf (stderr, "%s: can't read the database directory %s: %s\n",
 	       program_name, gnats_root, strerror (errno));
-      exit (1);
+      exit (3);
     }
 
   if (c == NULL || *c == '\0')
@@ -119,7 +121,7 @@ do_category (c)
   /* Process each file in the directory; ignore files that have periods
      in their names; either they're the . and .. dirs, or they're a
      lock file (1234.lock).  */
-  while (next = readdir (d))
+  while ((next = readdir (d)))
     if (strchr (next->d_name, '.') == NULL)
       {
 	p = path + len - 1;
@@ -222,7 +224,6 @@ no_entry:
 Categories *
 get_categories ()
 {
-  register char *p, *q;
   char *path = (char *) xmalloc (PATH_MAX);
   FILE *fp;
   Categories *cat_list = (Categories *) NULL;
@@ -237,7 +238,7 @@ get_categories ()
     {
       fprintf (stderr, "%s: couldn't read the categories file %s: %s\n",
 	       program_name, path, strerror (errno));
-      exit (1);
+      exit (3);
     }
 
   while ((c = next_category (fp)))
@@ -281,9 +282,9 @@ main (argc, argv)
 	  outfile = fopen (optarg, "w+");
 	  if (outfile == (FILE *) NULL)
 	    {
-	      fprintf (stderr, "%s: can't write to %s:", program_name,
+	      fprintf (stderr, "%s: can't write to %s: %s\n", program_name,
 		       optarg, strerror (errno));
-	      exit (1);
+	      exit (3);
 	    }
 	  break;
 
@@ -303,7 +304,9 @@ main (argc, argv)
 	}
     }
 
+  configure ();
   init_gnats ();
+  umask (022);
 
   if (sort_numerical)
     entries = (Entry *) xmalloc (max_entries * sizeof (Entry));
@@ -327,7 +330,7 @@ void
 usage ()
 {
   fprintf (stderr, "\
-Usage: %s [-hnv] [-d directory] [-o outfile] [--directory=directory]\n\
+Usage: %s [-hnV] [-d directory] [-o outfile] [--directory=directory]\n\
           [--outfile=filename] [--numerical] [--version] [--help]",
 	   program_name);
   exit (1);
