@@ -19,6 +19,8 @@ along with GNU GNATS; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "config.h"
+
 #if defined(__STDC__) || defined(_AIX)
 #include <stdarg.h>
 #define VA_START(args, lastarg) va_start(args, lastarg)
@@ -36,7 +38,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <sys/stat.h>
 
 #include "gnats.h"
-#include "config.h"
 #include "globals.h"
 #include "pathmax.h"
 
@@ -198,12 +199,19 @@ copy_file (source, dest)
 void 
 lock_gnats ()
 {
-
   char *path = (char *) xmalloc (PATH_MAX);
   struct stat buf;
+  int count;
 
   sprintf (path, "%s/gnats-adm/gnats.lock", gnats_root);
-  
+
+#define MAXWAIT 10
+#define GRANULARITY 1
+
+  for (count = 0 ;
+       count < MAXWAIT / GRANULARITY && stat (path, &buf) != -1 ;
+       sleep(GRANULARITY), ++count)
+    ;
   if (stat(path, &buf) != -1)
     {
       fprintf (stderr, "lock file exists %s\n", path);
@@ -213,7 +221,7 @@ lock_gnats ()
   if (creat (path, 0) == -1)
     {
       fprintf (stderr, "can not create lock file: %s\n", path);
-      exit (1);
+      exit (3);
     }
 }
 
@@ -226,16 +234,30 @@ unlock_gnats ()
   sprintf (path, "%s/gnats-adm/gnats.lock", gnats_root);
   
   if (stat (path, &buf) < 0)
-    exit (1);
+    return;
 
   if (unlink (path) < 0)
     {
       fprintf (stderr, "%s: cannot unlink lock file %s: %s\n",
 	       program_name, path, (char *) strerror (errno));
-      exit (1);
+      exit (3);
     }
 
 }
+
+int
+gnats_locked ()
+{
+  char *path = (char *) xmalloc (PATH_MAX);
+  struct stat buf;
+
+  sprintf (path, "%s/gnats-adm/gnats.lock", gnats_root);
+  
+  if (stat (path, &buf) < 0)
+    return 0;
+
+  return 1;
+}
 
 /* VARARGS */
 void 
@@ -255,17 +277,20 @@ punt (die, va_alist)
 
   fmt = va_arg (args, char *);
   
-  fp = open_mail_file (GNATS_ADMIN);
+  fp = open_mail_file (gnats_admin);
   if (fp == (FILE *) NULL)
     {
       fprintf (stderr, "can not open mail file for errors...panic!");
       unlock_gnats ();
-      exit (1);
+      exit (3);
     }
 
   fprintf (fp, "From: gnats (GNATS Management)\n");
+#ifndef USE_BIN_MAIL
+  fprintf (fp, "To: %s\n", gnats_admin);
+#endif
   fprintf (fp, "Subject: problem with %s\n", program_name);
-  fprintf (fp, "\n");
+  fprintf (fp, "\n\n");		/* Two newlines to massage buggy sendmails */
 
   vfprintf (fp, fmt, args);
 
@@ -276,20 +301,52 @@ punt (die, va_alist)
   if (die)
     {
       unlock_gnats ();
-      exit (1);
+      exit (2);
     }
 }
 
+void
+unlock_quit ()
+{
+  unlock_gnats ();
+  unblock_signals ();
+  exit (1);
+}
+
 void
 block_signals ()
 {
-  signal (SIGINT, SIG_IGN);
-  signal (SIGQUIT, SIG_IGN);
+  signal (SIGSEGV, unlock_quit);
+  signal (SIGHUP, unlock_quit);
+  signal (SIGTERM, unlock_quit);
+  signal (SIGINT, unlock_quit);
+  signal (SIGQUIT, unlock_quit);
+#ifdef SIGIOT
+  signal (SIGIOT, unlock_quit);
+#endif
+#ifdef SIGILL
+  signal (SIGILL, unlock_quit);
+#endif
+#ifdef SIGABRT
+  signal (SIGABRT, unlock_quit);
+#endif
 }
 
 void
 unblock_signals ()
 {
+  signal (SIGSEGV, SIG_DFL);
+  signal (SIGHUP, SIG_DFL);
+  signal (SIGTERM, SIG_DFL);
   signal (SIGINT, SIG_DFL);
   signal (SIGQUIT, SIG_DFL);
+#ifdef SIGIOT
+  signal (SIGIOT, SIG_DFL);
+#endif
+#ifdef SIGILL
+  signal (SIGILL, SIG_DFL);
+#endif
+#ifdef SIGABRT
+  signal (SIGABRT, SIG_DFL);
+#endif
 }
