@@ -18,6 +18,8 @@ You should have received a copy of the GNU General Public License
 along with GNU GNATS; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#include "config.h"
+
 #include <stdio.h>
 #ifdef HAVE_SYSLOG_H
 #include <syslog.h>
@@ -31,7 +33,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define VA_START(args, lastarg) va_start(args)
 #endif
 
-#include "config.h"
 #include "gnats.h"
 #include "globals.h"
 #include "pathmax.h"
@@ -44,15 +45,9 @@ int debug_level = LOG_ERR;
 /* File to log all messages to.  */
 FILE *gnats_logfile = NULL;
 
-/* The program to be used to send the mail.  */
-char *mail_agent = MAIL_AGENT;
-
 /* Current logging method.  */
 Logging_Methods log_method = STDERR;
 
-/* Top of the GNATS tree.  */
-char *gnats_root = GNATS_ROOT;
-
 /* Table to tell if a char c is whitespace.  */
 unsigned char is_space[256];
 
@@ -103,7 +98,7 @@ log_msg (severity, has_arg, va_alist)
   va_end (args);
 
   if (severity == LOG_ERR)
-    exit (1);
+    exit (2);
 }
 
 /* Open the file (actually the pipe) to which the mail message will
@@ -116,19 +111,30 @@ open_mail_file (recipients)
   char *buffer;
 
   if (recipients != NULL)
-    buffer = (char *) xmalloc (strlen (MAIL_AGENT) + strlen (recipients) + 2);
+    buffer = (char *) xmalloc (strlen (mail_agent) + strlen (recipients) + 2);
   else
-    buffer = (char *) xmalloc (strlen (MAIL_AGENT) + 1);
+    buffer = (char *) xmalloc (strlen (mail_agent) + 1);
 
   /* Can't use log_msg here, since gnats_logfile is being set by this first
      thing.  */
 
 #ifdef USE_BIN_MAIL
-  sprintf (buffer, "%s %s", MAIL_AGENT, recipients);
+  sprintf (buffer, "%s %s", mail_agent,
+	   (debug_mode ? gnats_admin : recipients));
 #else
-  sprintf (buffer, "%s", MAIL_AGENT);
+  sprintf (buffer, "%s", mail_agent);
 #endif
-  fp = popen (buffer, "w+");
+  fp = popen (buffer, "w");
+
+  if (debug_mode)
+    {
+      fprintf (fp, "From: gnats (GNATS Management)\n");
+#ifndef USE_BIN_MAIL
+      fprintf (fp, "To: %s\n", gnats_admin);
+#endif
+      fprintf (fp, "Subject: mail output from %s\n", program_name);
+      fprintf (fp, "\n\n");
+    }
 
   return fp;
 }
@@ -167,7 +173,7 @@ init_space ()
 char *
 get_next_field (line, token, delim)
      char *line, *token;
-     char delim;
+     int delim;
 {
   char *t = token;
 
