@@ -20,8 +20,8 @@
 # the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 GNATS_ROOT=XGNATS_ROOTX
-VERSION=XVERSIONX
 GNATS_SITE=XGNATS_SITEX
+DATADIR=XDATADIRX
 
 FILES="COPYING README send-pr.sh install-sid.sh send-pr-el.in send-pr.texi \
        send-pr.man send-pr.info texinfo.tex categories"
@@ -31,13 +31,11 @@ if [ $# != 2 ]; then
   exit 1
 fi
  
-if [ ! -d $GNATS_ROOT ]; then
-  echo "$0: GNATS base directory $GNATS_ROOT does not exist."
-  exit 1
-fi
+# Newer config information?
+[ -f ${GNATS_ROOT}/gnats-adm/config ] && . ${GNATS_ROOT}/gnats-adm/config
 
-if [ ! -d $GNATS_ROOT/gnats-dist ]; then
-  echo "$0: GNATS send-pr distribution directory $GNATS_ROOT/gnats-dist does not exist."
+if [ ! -d ${DATADIR}/gnats/dist ]; then
+  echo "$0: GNATS send-pr distribution directory ${DATADIR}/gnats/dist does not exist."
   exit 1
 fi
 
@@ -47,24 +45,23 @@ if [ ! -d $1 ]; then
 fi
 
 for f in $FILES; do
-  if [ ! -f $GNATS_ROOT/gnats-dist/$f ]; then
-    echo "$0: could not copy $f to $1"
+  if [ ! -f ${DATADIR}/gnats/dist/$f ]; then
+    echo "$0: $f does not exist."
     exit 1
   fi
   rm -f $1/$f
-  cp $GNATS_ROOT/gnats-dist/$f $1
+  cp ${DATADIR}/gnats/dist/$f $1
 done
 
 cat > $1/Makefile << '__EOF__'
 #
 # Makefile for building a standalone send-pr.
 #
-VERSION=YVERSIONY
 RELEASE=YRELEASEY
 
 prefix = /usr/local
 bindir = $(prefix)/bin
-libdir = $(prefix)/lib
+datadir = $(prefix)/lib
 infodir = $(prefix)/info
 lispdir = $(prefix)/emacs/lisp
 mandir = $(prefix)/man
@@ -73,16 +70,15 @@ man1dir = $(mandir)/man1
 all: send-pr send-pr.el install-sid
 
 send-pr: send-pr.sh
-	sed -e 's/@VERSION@/$(VERSION)/g' \
+	sed -e 's,@DATADIR@,$(datadir),g' \
 	    -e 's/@DEFAULT_RELEASE@/$(RELEASE)/g' send-pr.sh > send-pr
 
 send-pr.el: send-pr-el.in
-	sed -e 's/@VERSION@/$(VERSION)/g' \
+	sed -e 's,@DATADIR@,$(datadir),g' \
 	    -e 's/@DEFAULT_RELEASE@/$(RELEASE)/g' send-pr-el.in > send-pr.el
 
 install-sid: install-sid.sh
-	sed -e 's/@VERSION@/$(VERSION)/g' \
-	    -e 's,@BINDIR@,$(bindir),g' install-sid.sh > install-sid
+	sed -e 's,@BINDIR@,$(bindir),g' install-sid.sh > install-sid
 
 install: all
 	if [ -d $(prefix) ]; then true ; else mkdir $(prefix) ; fi
@@ -94,23 +90,26 @@ install: all
 	-parent=`echo $(lispdir)|sed -e 's@/[^/]*$$@@'`; \
 	if [ -d $$parent ] ; then true ; else mkdir $$parent ; fi
 	if [ -d $(lispdir) ] ; then true ; else mkdir $(lispdir) ; fi
-	cp send-pr.el $(lispdir)
-	cp send-pr.info $(infodir)
-	if [ -d $(libdir) ] ; then true ; else mkdir $(libdir) ; fi
-	if [ -d $(libdir)/gnats ] ; then true ; else mkdir $(libdir)/gnats ; fi
-	cp categories $(libdir)/gnats/@GNATS_SITE@
+	cp send-pr.el $(lispdir)/send-pr.el
+	chmod 644 $(lispdir)/send-pr.el
+	cp send-pr.info $(infodir)/send-pr.info
+	chmod 644 $(infodir)/send-pr.info
+	if [ -d $(datadir) ] ; then true ; else mkdir $(datadir) ; fi
+	if [ -d $(datadir)/gnats ] ; then true ; else mkdir $(datadir)/gnats ; fi
+	cp categories $(datadir)/gnats/YGNATS_SITEY
+	chmod 644 $(datadir)/gnats/YGNATS_SITEY
 	-parent=`echo $(man1dir)|sed -e 's@/[^/]*$$@@'`; \
 	if [ -d $$parent ] ; then true ; else mkdir $$parent ; fi
 	if [ -d $(man1dir) ] ; then true ; else mkdir $(man1dir) ; fi
-	cp send-pr.1 $(man1dir)
+	cp send-pr.1 $(man1dir)/send-pr.1
+	chmod 644 $(man1dir)/send-pr.1
 
 clean:
 	rm -f send-pr send-pr.el
 
 __EOF__
 
-sed -e 's/YGNATS_SITEY/$GNATS_SITE/g' \
-    -e "s/YVERSIONY/$VERSION/1" \
+sed -e "s/YGNATS_SITEY/$GNATS_SITE/g" \
     -e "s/YRELEASEY/$2/" $1/Makefile > $1/Makefile.tmp
 mv -f $1/Makefile.tmp $1/Makefile
 
