@@ -40,6 +40,7 @@ typedef enum
   ENVIRONMENT,         /* Supplied by Sender */
   DESCRIPTION,         /* Supplied by Sender */
   HOW_TO_REPEAT,       /* Supplied by Sender */
+  FIX,		       /* Supplied by either */
   AUDIT_TRAIL,         /* Supplied by Receiver */
   UNFORMATTED,         /* Catch all field */
   NUM_PR_ITEMS	/* this needs to stay in */
@@ -101,5 +102,6 @@ typedef struct index_entry
 #define STATE_STRING 		">State:"
 #define SYNOPSIS_STRING 	">Synopsis:"
 #define UNFORMATTED_STRING 	">Unformatted:"
+#define FIX_STRING		">Fix:"
 
 #endif /* _pr_h_ */
